/* OL 8 specific tunables.
   Copyright (C) 2017 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>.  */

#include <string.h>
#include <unistd.h>
#include <ldsodefs.h>
#include <elision-conf.h>

const char *envname = "GLIBC_TUNABLES";

extern int __nptl_stack_hugetlb;

enum tunables {
  TUNABLE_INVALID = -1,
  TUNABLE_STACK_HUGETLB,
  TUNABLE_MAX
};

const char *tunables[TUNABLE_MAX] = {
  "glibc.pthread.stack_hugetlb"
};

static enum tunables
tunable_type_for_string(char *str)
{
  int i;
  if (str)
    for (i = 0; i < TUNABLE_MAX; i++)
      {
	if(strcmp(str, tunables[i]) == 0)
	  return (enum tunables) i;
      }
  return TUNABLE_INVALID;
}

static bool
parse_boolean_value(char *str)
{
  if(str[0] == '0' || str[0] == '\0')
    return false;
  else
    return true;
}

void
__nptl_tunables_init (void)
{
  char **ep = __environ;

  if (ep == NULL)
    return;

  for (; *ep != NULL; ep++)
    {
      char *value, *key;
      char *str = strdup(*ep);

      key = strtok(str, "=");
      if (strcmp(key, envname) != 0)
	continue;

      /* Set search point to first tunable variable entry.  */
      key = strtok(NULL, "=");
      value = strtok(NULL, ":");

      switch (tunable_type_for_string (key))
	{
	case TUNABLE_STACK_HUGETLB:
	  __nptl_stack_hugetlb = parse_boolean_value(value);
	  break;

	default:
	  break;
        }

      free(str);
    }
}
